/**
 * \class photomultiplier
 *
 * This class is base for class pmt and class sipm. Since it implements attributes and methods relative to both clases some of them are declared virtual so they can be
 * redefined if necessary.
 *
 * \author $Author: ne0/ac/akis $
 *
 * \version $Revision: 1.0 $
 *
 * \date $Date: $
 *
 * Contact: ealvarado@elo.utfsm.cl, spizarro@elo.utfsm.cl, gvalinm@elo.utfsm.cl
 *
 */

#ifndef _photomultiplier_H
#define	_photomultiplier_H

#include <TROOT.h>

#include "voltagesource.h"
#include "heatsource.h"
#include "sensor.h"
#include <cmath>
#include <cstdlib>
#include <ctime>
/**
 * \brief This class is Parent to sipm and pmt. It's used as a sensor during the simulated experiment
 */
class Photomultiplier {
public:
		///Void class constructor.
    Photomultiplier();
		///Class copy constructor.
    Photomultiplier(const Photomultiplier& orig);
		///Complete class constructor with all what is needed to create a "real" photomultiplier.
    Photomultiplier(Float_t _MagneticAngle, Float_t _MagneticMagnitude, VoltageSource & _InputVoltage, HeatSource & _InputTemperature, Float_t BreakdownVoltage);
		///Class destructor.
    virtual ~Photomultiplier();
		///Method used to set the magnetic field along the photomultiplier.
    void setMagneticField(Float_t _MagneticMagnitude, Float_t _MagneticAngle);
		///Method used to set the temperature source associated to the photomultiplier.
    void setTemperature(HeatSource &_InputTemperature);
		///Method used to set the voltage source associated to the photomultiplier.
    void setVoltage(VoltageSource &_InputVoltage);
		///Method to retrieve the current measure. Virtual to be rewritten in Child classes.
    virtual Float_t getMeasureCurrent();
		///Method to retrieve the voltage measure. Virtual to be rewritten in Child classes.
    virtual Float_t getMeasureVoltage();
		///Method to retrieve the breakdownvoltage of the sipm. Virtual to be rewritten in Child classes.
    virtual Float_t getBreakdownVoltage();
		/// Attribute needed to be public.
    Float_t BreakdownVoltage; 
private:
    /// Private method to set the gain of the photomultiplier
    void setGain();
    /// Private method to set the dark current of the photomultiplier
    void setDarkCurrent();
    /// Private method to get random numbers to generate different data for each different photomultiplier.
    Float_t getRandom(); 
    VoltageSource InputVoltage;      /// Stores the address of the object of voltage source used in this photomultiplier.
    HeatSource InputTemperature;     /// Stores the address of the object of heat source used in this photomultiplier.
    Float_t Seed;										 /// Seed needed to store random 
    Float_t DarkCurrent; 						 /// Darkcurrent of this photomultiplier.
    Float_t Gain_db;								 /// Gain in dB 
    Float_t MagneticAngle;					 /// Magnetic Angle relative to the position of photomultiplier.
    Float_t MagneticMagnitude;       /// Magnetic Magnitude of where it is this photomultipliers.

};

#endif	/* _photomultiplier_H */

